@@ -61,6 +61,7 @@ from .config import (
     )
 from .util import (
     debuild_config,
+    get_build_architecture,
     )
 
 dont_purge_opt = Option('dont-purge',
@@ -91,14 +92,6 @@ export_upstream_revision_opt = Option('export-upstream-revision',
     type=str, argname="REVISION")
 
 
-def changes_filename(package, version, arch):
-    non_epoch_version = version.upstream_version
-    if version.debian_version is not None:
-        non_epoch_version += "-%s" % version.debian_version
-    return "%s_%s_%s.changes" % (package,
-            non_epoch_version, arch)
-
-
 def _check_tree(tree, strict=False):
     if strict:
         for unknown in tree.unknowns():
@@ -111,14 +104,11 @@ def _check_tree(tree, strict=False):
             "You must resolve these before building.")
 
 
-def _get_build_architecture():
-    import subprocess
-    try:
-        return subprocess.check_output(
-            ['dpkg-architecture', '-qDEB_BUILD_ARCH']).strip()
-    except subprocess.CalledProcessError as e:
-        raise BzrCommandError(
-            "Could not find the build architecture: %s" % e)
+def _check_uncommitted(tree):
+    if tree.changes_from(tree.basis_tree()).has_changed():
+      raise BzrCommandError(gettext("There are uncommitted "
+              "changes in the working tree. You must commit "
+              "before using this command"))
 
 
 def _get_changelog_info(tree, last_version=None, package=None, distribution=None):
@@ -126,8 +116,6 @@ def _get_changelog_info(tree, last_version=None, package=None, distribution=None
         find_changelog,
         find_last_distribution,
         lookup_distribution,
-        )
-    from .errors import (
         MissingChangelogError,
         )
     DEFAULT_FALLBACK_DISTRIBUTION = "debian"
@@ -195,7 +183,7 @@ def _get_upstream_sources(local_tree, branch, build_type, config,
     from .upstream.branch import (
         LazyUpstreamBranchSource,
         )
-    yield PristineTarSource(branch)
+    yield PristineTarSource.from_tree(branch, local_tree)
     yield AptSource()
     if build_type == BUILD_TYPE_MERGE:
         upstream_branch_source = _get_upstream_branch_source(
@@ -294,8 +282,6 @@ class cmd_builddeb(Command):
     debian/rules binary'. It is overriden if --builder is passed. Using this
     and --reuse allows for fast rebuilds.
     """
-    working_tree_opt = Option('working-tree', help="This option has no effect.",
-                              short_name='w')
     export_only_opt = Option('export-only', help="Export only, don't build.",
                              short_name='e')
     use_existing_opt = Option('use-existing',
@@ -317,7 +303,7 @@ class cmd_builddeb(Command):
             "another source.")
     takes_args = ['branch_or_build_options*']
     aliases = ['bd', 'debuild']
-    takes_options = [working_tree_opt, export_only_opt,
+    takes_options = [export_only_opt,
         dont_purge_opt, use_existing_opt, result_opt, builder_opt, merge_opt,
         build_dir_opt, orig_dir_opt, split_opt,
         export_upstream_opt, export_upstream_revision_opt,
@@ -342,7 +328,6 @@ class cmd_builddeb(Command):
     def _get_build_tree(self, revision, tree, branch):
         if revision is None and tree is not None:
             note(gettext("Building using working tree"))
-            working_tree = True
         else:
             if revision is None:
                 revid = branch.last_revision()
@@ -354,8 +339,7 @@ class cmd_builddeb(Command):
                     'revision specifier.'))
             note(gettext("Building branch from revision %s"), revid)
             tree = branch.repository.revision_tree(revid)
-            working_tree = False
-        return tree, working_tree
+        return tree
 
     def _build_type(self, merge, native, split):
         if merge:
@@ -422,7 +406,6 @@ class cmd_builddeb(Command):
         return branch, build_options, source
 
     def run(self, branch_or_build_options_list=None, verbose=False,
-            working_tree=False,
             export_only=False, dont_purge=False, use_existing=False,
             result_dir=None, builder=None, merge=None, build_dir=None,
             export_upstream=None, export_upstream_revision=None,
@@ -430,7 +413,6 @@ class cmd_builddeb(Command):
             quick=False, reuse=False, native=None,
             source=False, revision=None, package_merge=None,
             strict=False):
-        import subprocess
         from .builder import DebBuild
         from .errors import (
             NoPreviousUpload,
@@ -440,18 +422,18 @@ class cmd_builddeb(Command):
             dget_changes,
             find_changelog,
             find_previous_upload,
-            guess_build_type,
             tree_contains_upstream_source,
+            changes_filename,
             )
 
         location, build_options, source = self._branch_and_build_options(
                 branch_or_build_options_list, source)
         tree, branch, is_local, location = self._get_tree_and_branch(location)
-        tree, is_working_tree = self._get_build_tree(revision, tree, branch)
+        tree = self._get_build_tree(revision, tree, branch)
         _check_tree(tree, strict=strict)
 
         with tree.lock_read():
-            config = debuild_config(tree, is_working_tree)
+            config = debuild_config(tree)
             if reuse:
                 note(gettext("Reusing existing build dir"))
                 dont_purge = True
@@ -505,7 +487,7 @@ class cmd_builddeb(Command):
                 if source:
                     arch = "source"
                 else:
-                    arch = _get_build_architecture()
+                    arch = get_build_architecture()
                 changes = changes_filename(changelog.package, changelog.version, arch)
                 changes_path = os.path.join(build_dir, changes)
                 if not os.path.exists(changes_path):
@@ -548,7 +530,7 @@ class cmd_get_orig_source(Command):
             find_changelog,
             )
         tree = WorkingTree.open_containing(directory)[0]
-        config = debuild_config(tree, tree)
+        config = debuild_config(tree)
 
         (changelog, larstiq) = find_changelog(tree, True)
         orig_dir = config.orig_dir
@@ -560,7 +542,7 @@ class cmd_get_orig_source(Command):
 
         upstream_provider = UpstreamProvider(changelog.package,
                 str(version), orig_dir,
-                [PristineTarSource(tree.branch),
+                [PristineTarSource.from_tree(tree.branch, tree),
                  AptSource(),
                  UScanSource(tree, larstiq) ])
 
@@ -643,10 +625,15 @@ class cmd_merge_upstream(Command):
     launchpad_opt = Option('launchpad',
         help='Use Launchpad to find upstream locations.')
 
+    force_pristine_tar_opt = Option(
+        'force-pristine-tar', help=(
+            'Force the use of pristine-tar, even if no '
+            'pristine-tar branch exists'))
+
     takes_options = [package_opt, version_opt,
             distribution_opt, directory_opt, last_version_opt,
             force_opt, 'revision', 'merge-type',
-            snapshot_opt, launchpad_opt]
+            snapshot_opt, launchpad_opt, force_pristine_tar_opt]
 
     def _add_changelog_entry(self, tree, package, version, distribution_name,
             changelog):
@@ -664,68 +651,20 @@ class cmd_merge_upstream(Command):
                     'merge had completed failed. Add the new changelog '
                     'entry yourself, review the merge, and then commit.')
 
-    def _do_merge(self, tree, tarball_filenames, package, version,
-            current_version, upstream_branch, upstream_revisions, merge_type,
-            force):
-        from .import_dsc import (
-            DistributionBranch,
-            DistributionBranchSet,
-            )
-        from .util import (
-            component_from_orig_tarball,
-            )
-        db = DistributionBranch(tree.branch, tree.branch, tree=tree)
-        dbs = DistributionBranchSet()
-        dbs.add_branch(db)
-        tarballs = [(p, component_from_orig_tarball(p, package, version)) for p
-                in tarball_filenames]
-        conflicts = db.merge_upstream(tarballs, package, version,
-                current_version, upstream_branch=upstream_branch,
-                upstream_revisions=upstream_revisions,
-                merge_type=merge_type, force=force)
-        return conflicts
-
-    def _fetch_tarball(self, package, version, orig_dir, locations, v3):
-        from .repack_tarball import repack_tarball
-        from .util import tarball_name
-        ret = []
-        format = None
-        for location in locations:
-            if v3:
-                if location.endswith(".tar.bz2") or location.endswith(".tbz2"):
-                    format = "bz2"
-                elif location.endswith(".tar.xz"):
-                    format = "xz"
-            dest_name = tarball_name(package, version, None, format=format)
-            tarball_filename = os.path.join(orig_dir, dest_name)
-            try:
-                repack_tarball(location, dest_name, target_dir=orig_dir)
-            except FileExists:
-                raise BzrCommandError("The target file %s already exists, and is either "
-                                      "different to the new upstream tarball, or they "
-                                      "are of different formats. Either delete the target "
-                                      "file, or use it as the argument to import."
-                                      % dest_name)
-            ret.append(tarball_filename)
-        return ret
-
-    def _get_tarballs(self, config, tree, package, version, upstream_branch,
-            upstream_revision, v3, locations):
-        orig_dir = config.orig_dir or default_orig_dir
-        orig_dir = os.path.join(tree.basedir, orig_dir)
-        if not os.path.exists(orig_dir):
-            os.makedirs(orig_dir)
-        return self._fetch_tarball(package, version, orig_dir,
-            locations, v3)
-
     def run(self, location=None, upstream_branch=None, version=None,
             distribution=None, package=None,
             directory=".", revision=None, merge_type=None,
-            last_version=None, force=None, snapshot=False, launchpad=False):
+            last_version=None, force=None, snapshot=False, launchpad=False,
+            force_pristine_tar=False):
         from debian.changelog import Version
 
         from .errors import PackageVersionNotPresent
         from .hooks import run_hook
+        from .merge_upstream import (
+            do_merge,
+            get_tarballs,
+            PreviousVersionTagMissing,
+            )
         from .upstream import (
             TarfileSource,
             UScanSource,
@@ -734,21 +673,14 @@ class cmd_merge_upstream(Command):
             UpstreamBranchSource,
             )
         from .util import (
-            FORMAT_3_0_QUILT,
-            FORMAT_3_0_NATIVE,
-            tree_get_source_format,
             guess_build_type,
             tree_contains_upstream_source,
             )
 
         tree, _ = WorkingTree.open_containing(directory)
         with tree.lock_write():
-            # Check for uncommitted changes.
-            if tree.changes_from(tree.basis_tree()).has_changed():
-                raise BzrCommandError("There are uncommitted changes in the "
-                        "working tree. You must commit before using this "
-                        "command.")
-            config = debuild_config(tree, tree)
+            _check_uncommitted(tree)
+            config = debuild_config(tree)
             (current_version, package, distribution, distribution_name,
              changelog, top_level) = _get_changelog_info(tree, last_version,
                  package, distribution)
@@ -869,15 +801,25 @@ class cmd_merge_upstream(Command):
                             revisions=upstream_revisions)
                     else:
                         raise
-                source_format = tree_get_source_format(tree)
-                v3 = (source_format in [
-                    FORMAT_3_0_QUILT, FORMAT_3_0_NATIVE])
-                tarball_filenames = self._get_tarballs(config, tree, package,
-                    version, upstream_branch, upstream_revisions, v3,
-                    locations)
-                conflicts = self._do_merge(tree, tarball_filenames, package,
-                    version, current_version, upstream_branch, upstream_revisions,
-                    merge_type, force)
+                orig_dir = config.orig_dir or default_orig_dir
+                try:
+                    tarball_filenames = get_tarballs(orig_dir, tree, package,
+                        version, upstream_branch, upstream_revisions,
+                        locations)
+                except FileExists as e:
+                    raise BzrCommandError(
+                        "The target file %s already exists, and is either "
+                        "different to the new upstream tarball, or they "
+                        "are of different formats. Either delete the target "
+                        "file, or use it as the argument to import."
+                        % e.path)
+                try:
+                    conflicts = do_merge(tree, tarball_filenames, package,
+                        version, current_version, upstream_branch, upstream_revisions,
+                        merge_type, force=force,
+                        force_pristine_tar=force_pristine_tar)
+                except PreviousVersionTagMissing as e:
+                    raise BzrCommandError(str(e))
             if (current_version is not None and
                 Version(current_version) >= Version(version)):
                 raise BzrCommandError(
@@ -959,9 +901,6 @@ class cmd_import_dsc(Command):
             db.import_package(os.path.join(orig_target, filename))
 
     def run(self, files_list, file=None):
-        from .errors import (
-            MissingChangelogError,
-            )
         from .import_dsc import (
             DistributionBranch,
             DistributionBranchSet,
@@ -969,6 +908,7 @@ class cmd_import_dsc(Command):
         from .util import (
             find_changelog,
             open_file,
+            MissingChangelogError,
             )
         try:
             tree = WorkingTree.open_containing('.')[0]
@@ -976,10 +916,7 @@ class cmd_import_dsc(Command):
             raise BzrCommandError(gettext(
                 "There is no tree to import the packages in to"))
         with tree.lock_write():
-            if tree.changes_from(tree.basis_tree()).has_changed():
-                raise BzrCommandError(gettext("There are uncommitted "
-                        "changes in the working tree. You must commit "
-                        "before using this command"))
+            _check_uncommitted(tree)
             if files_list is None:
                 files_list = []
             if file is not None:
@@ -994,7 +931,7 @@ class cmd_import_dsc(Command):
                 raise BzrCommandError(gettext("You must give the location of "
                     "at least one source package to install, or use the "
                     "--file option."))
-            config = debuild_config(tree, tree)
+            config = debuild_config(tree)
             if config.build_type == BUILD_TYPE_MERGE:
                 raise BzrCommandError(
                     gettext("import-dsc in merge mode is not yet supported."))
@@ -1068,11 +1005,17 @@ class cmd_import_upstream(Command):
     import and the tip of the upstream branch if you supply one.
     """
 
-    takes_options = ['revision']
+    takes_options = [
+        'revision',
+        Option(
+            'force-pristine-tar',
+            help=('Force creation of a new '
+                  'pristine-tar branch, even if one does not exist.'))]
 
     takes_args = ['version', 'location', 'upstream_branch?']
 
-    def run(self, version, location, upstream_branch=None, revision=None):
+    def run(self, version, location, upstream_branch=None, revision=None,
+            force_pristine_tar=False):
         from debian.changelog import Version
         from .import_dsc import (
             DistributionBranch,
@@ -1130,7 +1073,8 @@ class cmd_import_upstream(Command):
         tarballs = [(location, None, md5sum_filename(location))]
         for (component, tag_name, revid) in db.import_upstream_tarballs(
                 tarballs, None, version, parents, upstream_branch=upstream,
-                upstream_revisions={ None: upstream_revid }):
+                upstream_revisions={None: upstream_revid},
+                force_pristine_tar=force_pristine_tar):
             if component is None:
                 self.outf.write(gettext(
                     'Imported %(location)s as tag:%(tag)s.\n') % {
@@ -1173,13 +1117,11 @@ class cmd_builddeb_do(Command):
 
     def run(self, command_list=None):
         import subprocess
-        from .errors import (
-            BuildFailedError,
-            )
         from .source_distiller import (
             MergeModeDistiller,
             )
         from .builder import (
+            BuildFailedError,
             DebBuild,
             )
         from .upstream import (
@@ -1198,7 +1140,7 @@ class cmd_builddeb_do(Command):
             )
         t = WorkingTree.open_containing('.')[0]
         self.add_cleanup(t.lock_read().unlock)
-        config = debuild_config(t, t)
+        config = debuild_config(t)
         (changelog, top_level) = find_changelog(t, False, max_blocks=2)
 
         contains_upstream_source = tree_contains_upstream_source(t)
@@ -1232,7 +1174,7 @@ class cmd_builddeb_do(Command):
 
         upstream_provider = UpstreamProvider(changelog.package,
                 changelog.version.upstream_version, orig_dir,
-                [PristineTarSource(t.branch),
+                [PristineTarSource.from_tree(t.branch, t),
                  AptSource(),
                  UScanSource(t, top_level) ])
 
@@ -1300,11 +1242,8 @@ class cmd_mark_uploaded(Command):
             )
         t = WorkingTree.open_containing('.')[0]
         with t.lock_write():
-            if t.changes_from(t.basis_tree()).has_changed():
-              raise BzrCommandError(gettext("There are uncommitted "
-                      "changes in the working tree. You must commit "
-                      "before using this command"))
-            config = debuild_config(t, t)
+            _check_uncommitted(t)
+            config = debuild_config(t)
             if merge is None:
                 merge = (config.build_type == BUILD_TYPE_MERGE)
             (changelog, top_level) = find_changelog(t, merge)
@@ -1356,18 +1295,22 @@ class cmd_dh_make(Command):
     takes_args = ['package_name', 'version', 'tarball']
 
     bzr_only_opt = Option('bzr-only', help="Don't run dh_make.")
+    use_pristine_tar_opt = Option(
+        'use-pristine-tar', help='Whether to use pristine-tar.')
 
     takes_options = [bzr_only_opt]
 
-    def run(self, package_name, version, tarball, bzr_only=None):
+    def run(self, package_name, version, tarball, bzr_only=None,
+            use_pristine_tar=True):
         from . import dh_make
-        tree = dh_make.import_upstream(tarball, package_name,
-            version.encode("utf-8"))
+        tree = dh_make.import_upstream(
+            tarball, package_name, version.encode("utf-8"),
+            use_pristine_tar=use_pristine_tar)
         if not bzr_only:
             with tree.lock_write():
                 dh_make.run_dh_make(tree, package_name, version)
         note(gettext('Package prepared in %s'),
-            urlutils.unescape_for_display(tree.basedir, self.outf.encoding))
+             urlutils.unescape_for_display(tree.basedir, self.outf.encoding))
 
 
 class cmd_dep3_patch(Command):
@@ -1449,7 +1392,7 @@ class cmd_dep3_patch(Command):
         origin = describe_origin(branch, revision_id)
         if packaging_tree is None:
             packaging_tree = packaging_branch.basis_tree()
-        builddeb_config = debuild_config(packaging_tree, True)
+        builddeb_config = debuild_config(packaging_tree)
         if not no_upstream_check and builddeb_config.upstream_branch:
             upstream_branch = Branch.open(builddeb_config.upstream_branch)
             applied_upstream = determine_applied_upstream(upstream_branch,
@@ -1499,10 +1442,9 @@ class LocalTree(object):
 
 def _build_helper(local_tree, branch, target_dir, builder):
     # TODO(jelmer): Integrate this with cmd_builddeb
-    import subprocess
     from .hooks import run_hook
     from .builder import (
-        DebBuild,
+        do_build,
         )
     from .util import (
         find_changelog,
@@ -1513,7 +1455,7 @@ def _build_helper(local_tree, branch, target_dir, builder):
     (changelog, top_level) = find_changelog(
         local_tree, False, max_blocks=2)
 
-    config = debuild_config(local_tree, local_tree)
+    config = debuild_config(local_tree)
     contains_upstream_source = tree_contains_upstream_source(local_tree)
 
     distiller = _get_distiller(
@@ -1522,29 +1464,8 @@ def _build_helper(local_tree, branch, target_dir, builder):
             contains_upstream_source=contains_upstream_source,
             top_level=top_level)
 
-    bd = tempfile.mkdtemp()
-    try:
-        build_source_dir = os.path.join(bd,
-                changelog.package + "-" + changelog.version.upstream_version)
-        builder = DebBuild(
-                distiller, build_source_dir,
-                builder,
-                use_existing=False)
-        builder.prepare()
-        run_hook(local_tree, 'pre-export', config)
-        builder.export()
-        run_hook(local_tree, 'pre-build', config, wd=build_source_dir)
-        builder.build()
-        run_hook(local_tree, 'post-build', config, wd=build_source_dir)
-        changes = changes_filename(changelog.package, changelog.version, 'source')
-        changes_path = os.path.join(bd, changes)
-        if target_dir is not None:
-            if not os.path.exists(changes_path):
-                raise BzrCommandError("Could not find the .changes "
-                        "file from the build: %s" % changes_path)
-            return dget_changes(changes_path, target_dir)
-    finally:
-        shutil.rmtree(bd)
+    return do_build(changelog.package, changelog.version, distiller,
+                    local_tree, config, builder, target_dir)
 
 
 class cmd_debrelease(Command):
