@@ -71,6 +71,10 @@ class UpstreamMetadataConfig(object):
     if section == "BUILDDEB":
       if option == "upstream-branch":
         return self.metadata.get('Repository')
+      if option == "export-upstream-revision":
+        tag_prefix = self.metadata.get("Repository-Tag-Prefix")
+        if tag_prefix is not None:
+          return "tag:" + tag_prefix + "$UPSTREAM_VERSION"
     raise KeyError
 
   def __getitem__(self, key):
@@ -329,18 +333,6 @@ class DebBuildConfig(object):
   export_upstream_revision = _opt_property('export-upstream-revision',
                          "The revision of the upstream source to use.")
 
-  commit_message_from_changelog = _bool_property('commit-message-from-changelog',
-       "Whether the commit message should come from debian/changelog", default=False)
-
-  quilt_smart_merge = _bool_property('quilt-smart-merge',
-    "Automatically unapply quilt patches during merge", default=True)
-
-  quilt_tree_policy = _opt_property('quilt-tree-policy',
-      "Whether to automatically apply/unapply quilt patches after tree operations")
-
-  quilt_commit_policy = _opt_property("quilt-commit-policy",
-        "Policy for committing quilt patches (applied / unapplied)")
-
 
 def _test():
   import doctest
