@@ -30,11 +30,6 @@ class DebianError(BzrError):
         BzrError.__init__(self, cause=cause)
 
 
-class NoSourceDirError(BzrError):
-    _fmt = ("There is no existing source directory to use. Use "
-            "--export-only or --dont-purge to get one that can be used")
-
-
 class MissingUpstreamTarball(BzrError):
     _fmt = ("Unable to find the needed upstream tarball for package %(package)s, "
             "version %(version)s.")
@@ -50,10 +45,6 @@ class TarFailed(BzrError):
         BzrError.__init__(self, operation=operation, tarball=tarball)
 
 
-class BuildFailedError(BzrError):
-    _fmt = "The build failed."
-
-
 class UnparseableChangelog(BzrError):
     _fmt = "There was an error parsing the changelog: %(error)s"
 
@@ -68,13 +59,6 @@ class StopBuild(BzrError):
         BzrError.__init__(self, reason=reason)
 
 
-class MissingChangelogError(BzrError):
-    _fmt = 'Could not find changelog at %(location)s in tree.'
-
-    def __init__(self, locations):
-        BzrError.__init__(self, location=locations)
-
-
 class AddChangelogError(BzrError):
     _fmt = 'Please add "%(changelog)s" to the branch using bzr add.'
 
@@ -89,13 +73,6 @@ class ImportError(BzrError):
         BzrError.__init__(self, reason=reason)
 
 
-class HookFailedError(BzrError):
-    _fmt = 'The "%(hook_name)s" hook failed.'
-
-    def __init__(self, hook_name):
-        BzrError.__init__(self, hook_name=hook_name)
-
-
 class OnlyImportSingleDsc(BzrError):
     _fmt = "You are only allowed to import one version in incremental mode."
 
@@ -228,13 +205,4 @@ class MultipleUpstreamTarballsNotSupported(BzrError):
             "is not yet supported.")
 
 
-class QuiltUnapplyError(BzrError):
 
-    _fmt = ("Unable to unapply quilt patches for %(kind)r tree: %(msg)s")
-
-    def __init__(self, kind, msg):
-        BzrError.__init__(self)
-        self.kind = kind
-        if msg is not None and msg.count("\n") == 1:
-            msg = msg.strip()
-        self.msg = msg
