@@ -31,7 +31,6 @@ import calendar
 import os
 import shutil
 import stat
-import subprocess
 import tempfile
 
 from debian import deb822
@@ -46,8 +45,8 @@ from ...export import (
     )
 from ...config import ConfigObj
 from ...errors import (
+    BzrError,
     AlreadyBranchError,
-    BzrCommandError,
     NotBranchError,
     NoWorkingTree,
     UnrelatedBranches,
@@ -66,24 +65,31 @@ from .errors import (
     UpstreamAlreadyImported,
     UpstreamBranchAlreadyMerged,
     )
+from .extract import extract
 from .util import (
-    FORMAT_1_0,
-    FORMAT_3_0_QUILT,
-    FORMAT_3_0_NATIVE,
-    component_from_orig_tarball,
     extract_orig_tarballs,
     get_commit_info_from_changelog,
     md5sum_filename,
     open_file_via_transport,
     open_transport,
     safe_decode,
-    subprocess_setup,
     )
 from .upstream.pristinetar import (
     PristineTarSource,
     )
 
 
+class PreviousVersionTagMissing(BzrError):
+
+    _fmt = ("Unable to find the tag for the "
+            "previous upstream version (%(version)s) in the branch: "
+            "%(tag_name)s")
+
+    def __init__(self, version, tag_name):
+        super(PreviousVersionTagMissing, self).__init__(
+            version=version, tag_name=tag_name)
+
+
 class DscCache(object):
 
     def __init__(self, transport=None):
@@ -242,8 +248,8 @@ class DistributionBranch(object):
         self.pristine_upstream_branch = pristine_upstream_branch
         self.pristine_upstream_tree = pristine_upstream_tree
         if pristine_upstream_branch is not None:
-            self.pristine_upstream_source = PristineTarSource(
-                branch=pristine_upstream_branch)
+            self.pristine_upstream_source = PristineTarSource.from_tree(
+                branch=pristine_upstream_branch, tree=tree)
         else:
             self.pristine_upstream_source = None
         self.get_lesser_branches = None
@@ -780,7 +786,7 @@ class DistributionBranch(object):
     def import_upstream(self, upstream_part, package, version, upstream_parents,
             upstream_tarballs, upstream_branch=None,
             upstream_revisions=None, timestamp=None, author=None,
-            file_ids_from=None):
+            file_ids_from=None, force_pristine_tar=False, committer=None):
         """Import an upstream part on to the upstream branch.
 
         This imports the upstream part of the code and places it on to
@@ -855,7 +861,8 @@ class DistributionBranch(object):
             (tag, revid) = self.pristine_upstream_source.import_component_tarball(
                 package, version, self.pristine_upstream_tree, parents,
                 component, md5, tarball, author=author, timestamp=timestamp,
-                exclude=exclude)
+                exclude=exclude, force_pristine_tar=force_pristine_tar,
+                committer=committer)
             self.pristine_upstream_branch.generate_revision_history(revid)
             ret.append((component, tag, revid))
             self.branch.fetch(self.pristine_upstream_branch)
@@ -863,7 +870,7 @@ class DistributionBranch(object):
         return ret
 
     def import_upstream_tarballs(self, tarballs, package, version, parents,
-        upstream_branch=None, upstream_revisions=None):
+        upstream_branch=None, upstream_revisions=None, force_pristine_tar=False):
         """Import an upstream part to the upstream branch.
 
         :param tarballs: List of tarballs / components to extract
@@ -882,7 +889,8 @@ class DistributionBranch(object):
             return self.import_upstream(tarball_dir, package, version, parents,
                 tarballs,
                 upstream_branch=upstream_branch,
-                upstream_revisions=upstream_revisions)
+                upstream_revisions=upstream_revisions,
+                force_pristine_tar=force_pristine_tar)
         finally:
             shutil.rmtree(tarball_dir)
 
@@ -1080,7 +1088,7 @@ class DistributionBranch(object):
 
     def _import_normal_package(self, package, version, versions, debian_part, md5,
             upstream_part, upstream_tarballs, timestamp=None, author=None,
-            file_ids_from=None, pull_debian=True):
+            file_ids_from=None, pull_debian=True, force_pristine_tar=False):
         """Import a source package.
 
         :param package: Package name
@@ -1128,7 +1136,8 @@ class DistributionBranch(object):
                             upstream_parents,
                             upstream_tarballs=upstream_tarballs,
                             timestamp=timestamp, author=author,
-                            file_ids_from=file_ids_from)
+                            file_ids_from=file_ids_from,
+                            force_pristine_tar=force_pristine_tar)
                     self._fetch_upstream_to_branch(imported_revids)
             else:
                 mutter("We already have the needed upstream part")
@@ -1181,7 +1190,7 @@ class DistributionBranch(object):
         return versions
 
     def import_package(self, dsc_filename, use_time_from_changelog=True,
-            file_ids_from=None, pull_debian=True):
+            file_ids_from=None, pull_debian=True, force_pristine_tar=False):
         """Import a source package.
 
         :param dsc_filename: a path to a .dsc file for the version
@@ -1192,14 +1201,7 @@ class DistributionBranch(object):
         base_path = osutils.dirname(dsc_filename)
         dsc = deb822.Dsc(open(dsc_filename).read())
         version = Version(dsc['Version'])
-        format = dsc.get('Format', FORMAT_1_0).strip()
-        extractor_cls = SOURCE_EXTRACTORS.get(format)
-        if extractor_cls is None:
-            raise AssertionError("Don't know how to import source format %s yet"
-                    % format)
-        extractor = extractor_cls(dsc_filename, dsc)
-        try:
-            extractor.extract()
+        with extract(dsc_filename, dsc) as extractor:
             cl = self.get_changelog_from_source(extractor.extracted_debianised)
             timestamp = None
             author = None
@@ -1228,15 +1230,14 @@ class DistributionBranch(object):
                         extractor.upstream_tarballs,
                         timestamp=timestamp, author=author,
                         file_ids_from=file_ids_from,
-                        pull_debian=pull_debian)
+                        pull_debian=pull_debian,
+                        force_pristine_tar=force_pristine_tar)
             else:
                 self._import_native_package(dsc['Source'], version, versions,
                         extractor.extracted_debianised,
                         extractor.unextracted_debian_md5,
                         timestamp=timestamp, file_ids_from=file_ids_from,
                         pull_debian=pull_debian)
-        finally:
-            extractor.cleanup()
 
     def extract_upstream_tree(self, upstream_tips, basedir):
         """Extract upstream_tip to a tempdir as a working tree."""
@@ -1306,11 +1307,9 @@ class DistributionBranch(object):
             upstream_tips = self.pristine_upstream_source.version_as_revisions(
                     package, previous_version)
         except PackageVersionNotPresent:
-            raise BzrCommandError("Unable to find the tag for the "
-                    "previous upstream version (%s) in the branch: "
-                    "%s" % (
+            raise PreviousVersionTagMissing(
                 previous_version,
-                self.pristine_upstream_source.tag_name(previous_version)))
+                self.pristine_upstream_source.tag_name(previous_version))
         self.extract_upstream_tree(upstream_tips, tempdir)
 
     def has_merged_upstream_revisions(self, this_revision, upstream_repository, upstream_revisions):
@@ -1321,7 +1320,7 @@ class DistributionBranch(object):
 
     def merge_upstream(self, tarball_filenames, package, version, previous_version,
             upstream_branch=None, upstream_revisions=None, merge_type=None,
-            force=False):
+            force=False, force_pristine_tar=False, committer=None):
         tempdir = tempfile.mkdtemp(dir=os.path.join(self.tree.basedir, '..'))
         try:
             if previous_version is not None:
@@ -1353,7 +1352,9 @@ class DistributionBranch(object):
                             package, version, parents,
                             upstream_tarballs=upstream_tarballs,
                             upstream_branch=upstream_branch,
-                            upstream_revisions=upstream_revisions)
+                            upstream_revisions=upstream_revisions,
+                            force_pristine_tar=force_pristine_tar,
+                            committer=committer)
                     self._fetch_upstream_to_branch(imported_revids)
                 finally:
                     shutil.rmtree(tarball_dir)
@@ -1386,139 +1387,3 @@ class DistributionBranch(object):
         finally:
             shutil.rmtree(tempdir)
 
-
-class SourceExtractor(object):
-    """A class to extract a source package to its constituent parts"""
-
-    def __init__(self, dsc_path, dsc):
-        self.dsc_path = dsc_path
-        self.dsc = dsc
-        self.extracted_upstream = None
-        self.extracted_debianised = None
-        self.unextracted_debian_md5 = None
-        self.upstream_tarballs = []
-        self.tempdir = None
-
-    def extract(self):
-        """Extract the package to a new temporary directory."""
-        raise NotImplementedError(self.extract)
-
-    def cleanup(self):
-        """Cleanup any extracted files."""
-        if self.tempdir is not None and os.path.isdir(self.tempdir):
-            shutil.rmtree(self.tempdir)
-
-
-class OneZeroSourceExtractor(SourceExtractor):
-    """Source extract for the "1.0" source format."""
-
-    def extract(self):
-        """Extract the package to a new temporary directory."""
-        self.tempdir = tempfile.mkdtemp()
-        dsc_filename = os.path.abspath(self.dsc_path)
-        proc = subprocess.Popen("dpkg-source -su -x %s" % (dsc_filename,), shell=True,
-                cwd=self.tempdir, stdout=subprocess.PIPE,
-                stderr=subprocess.STDOUT, preexec_fn=subprocess_setup)
-        (stdout, _) = proc.communicate()
-        assert proc.returncode == 0, "dpkg-source -x failed, output:\n%s" % \
-                    (stdout,)
-        name = self.dsc['Source']
-        version = Version(self.dsc['Version'])
-        self.extracted_upstream = os.path.join(self.tempdir,
-                "%s-%s.orig" % (name, str(version.upstream_version)))
-        self.extracted_debianised = os.path.join(self.tempdir,
-                "%s-%s" % (name, str(version.upstream_version)))
-        if not os.path.exists(self.extracted_upstream):
-            mutter("It's a native package")
-            self.extracted_upstream = None
-        for part in self.dsc['files']:
-            if self.extracted_upstream is None:
-                if part['name'].endswith(".tar.gz"):
-                    self.unextracted_debian_md5 = part['md5sum']
-            else:
-                if part['name'].endswith(".orig.tar.gz"):
-                    self.upstream_tarballs.append((os.path.abspath(
-                            os.path.join(osutils.dirname(self.dsc_path),
-                                part['name'])),
-                            component_from_orig_tarball(part['name'], name, str(version.upstream_version)),
-                            str(part['md5sum'])))
-                elif part['name'].endswith(".diff.gz"):
-                    self.unextracted_debian_md5 = part['md5sum']
-
-
-class ThreeDotZeroNativeSourceExtractor(SourceExtractor):
-    """Source extractor for the "3.0 (native)" source format."""
-
-    def extract(self):
-        self.tempdir = tempfile.mkdtemp()
-        dsc_filename = os.path.abspath(self.dsc_path)
-        proc = subprocess.Popen("dpkg-source -x %s" % (dsc_filename,), shell=True,
-                cwd=self.tempdir, stdout=subprocess.PIPE,
-                stderr=subprocess.STDOUT, preexec_fn=subprocess_setup)
-        (stdout, _) = proc.communicate()
-        assert proc.returncode == 0, "dpkg-source -x failed, output:\n%s" % \
-                    (stdout,)
-        name = self.dsc['Source']
-        version = Version(self.dsc['Version'])
-        self.extracted_debianised = os.path.join(self.tempdir,
-                "%s-%s" % (name, str(version.upstream_version)))
-        self.extracted_upstream = None
-        for part in self.dsc['files']:
-            if (part['name'].endswith(".tar.gz")
-                    or part['name'].endswith(".tar.bz2")
-                    or part['name'].endswith(".tar.xz")):
-                self.unextracted_debian_md5 = part['md5sum']
-
-
-class ThreeDotZeroQuiltSourceExtractor(SourceExtractor):
-    """Source extractor for the "3.0 (quilt)" source format."""
-
-    def extract(self):
-        self.tempdir = tempfile.mkdtemp()
-        dsc_filename = os.path.abspath(self.dsc_path)
-        proc = subprocess.Popen("dpkg-source --skip-debianization -x %s"
-                % (dsc_filename,), shell=True,
-                cwd=self.tempdir, stdout=subprocess.PIPE,
-                stderr=subprocess.STDOUT, preexec_fn=subprocess_setup)
-        (stdout, _) = proc.communicate()
-        assert proc.returncode == 0, "dpkg-source -x failed, output:\n%s" % \
-                    (stdout,)
-        name = self.dsc['Source']
-        version = Version(self.dsc['Version'])
-        self.extracted_debianised = os.path.join(self.tempdir,
-                "%s-%s" % (name, str(version.upstream_version)))
-        self.extracted_upstream = self.extracted_debianised + ".orig"
-        os.rename(self.extracted_debianised, self.extracted_upstream)
-        proc = subprocess.Popen("dpkg-source -x %s" % (dsc_filename,), shell=True,
-                cwd=self.tempdir, stdout=subprocess.PIPE,
-                stderr=subprocess.STDOUT, preexec_fn=subprocess_setup)
-        (stdout, _) = proc.communicate()
-        assert proc.returncode == 0, "dpkg-source -x failed, output:\n%s" % \
-                    (stdout,)
-        # Check that there are no unreadable files extracted.
-        subprocess.call(["find", self.extracted_upstream, "-perm",
-                "0000", "-exec", "chmod", "644", "{}", ";"])
-        subprocess.call(["find", self.extracted_debianised, "-perm",
-                "0000", "-exec", "chmod", "644", "{}", ";"])
-        for part in self.dsc['files']:
-            if part['name'].startswith("%s_%s.orig" % (name, str(version.upstream_version))):
-                self.upstream_tarballs.append((
-                    os.path.abspath(os.path.join(osutils.dirname(self.dsc_path),
-                                    part['name'])),
-                    component_from_orig_tarball(part['name'], name,
-                        str(version.upstream_version)),
-                    str(part['md5sum'])))
-            elif (part['name'].endswith(".debian.tar.gz")
-                    or part['name'].endswith(".debian.tar.bz2")
-                    or part['name'].endswith(".debian.tar.xz")):
-                self.unextracted_debian_md5 = str(part['md5sum'])
-        assert self.upstream_tarballs is not None, \
-            "Can't handle non gz|bz2|xz tarballs yet"
-        assert self.unextracted_debian_md5 is not None, \
-            "Can't handle non gz|bz2|xz tarballs yet"
-
-
-SOURCE_EXTRACTORS = {}
-SOURCE_EXTRACTORS[FORMAT_1_0] = OneZeroSourceExtractor
-SOURCE_EXTRACTORS[FORMAT_3_0_NATIVE] = ThreeDotZeroNativeSourceExtractor
-SOURCE_EXTRACTORS[FORMAT_3_0_QUILT] = ThreeDotZeroQuiltSourceExtractor
