@@ -27,7 +27,6 @@ from ...errors import (
 from ...revisionspec import RevisionSpec, RevisionInfo
 
 from .errors import (
-    MissingChangelogError,
     UnknownVersion,
     VersionNotSpecified,
     )
@@ -45,7 +44,7 @@ class RevisionSpec_package(RevisionSpec):
     wants_revision_history = False
     prefix = 'package:'
 
-    def _match_on(self, branch, revs):
+    def _match_on(self, branch, revs=None):
         version_spec = self.spec
         dist_spec = None
 
@@ -72,9 +71,12 @@ class RevisionSpec_upstream(RevisionSpec):
     wants_revision_history = False
     prefix = 'upstream:'
 
-    def _match_on(self, branch, revs):
+    def _match_on(self, branch, revs=None):
         from ...workingtree import WorkingTree
-        from .util import find_changelog
+        from .util import (
+            find_changelog,
+            MissingChangelogError,
+            )
         from .upstream.pristinetar import PristineTarSource
         from debian.changelog import Version
         tree = WorkingTree.open_containing('.')[0]
