@@ -122,14 +122,14 @@ def load_tests(loader, basic_tests, pattern):
             'test_commit_message',
             'test_config',
             'test_dep3',
+            'test_directory',
             'test_dh_make',
+            'test_extract',
             'test_hooks',
             'test_import_dsc',
             'test_merge_changelog',
             'test_merge_package',
-            'test_merge_quilt',
             'test_merge_upstream',
-            'test_quilt',
             'test_repack_tarball_extra',
             'test_revspec',
             'test_source_distiller',
