@@ -53,10 +53,6 @@ def revision_history(branch):
 from ..import_dsc import (
         DistributionBranch,
         DistributionBranchSet,
-        OneZeroSourceExtractor,
-        SOURCE_EXTRACTORS,
-        ThreeDotZeroNativeSourceExtractor,
-        ThreeDotZeroQuiltSourceExtractor,
         )
 from ..upstream.pristinetar import (
         PristineTarDeltaTooLarge,
@@ -1872,174 +1868,3 @@ package (1.0-1) UNRELEASED; urgency=low
  -- Jo\xe9 Master <joe@example.com>  Thu,  2 Dec 2004 16:59:43 +0100
 '''.encode('latin_1'))])
         cl = self.db1.get_changelog_from_source(self.db1.tree.basedir)
-
-
-class OneZeroSourceExtractorTests(tests.TestCaseInTempDir):
-
-    def test_extract_format1(self):
-        version = Version("0.1-1")
-        name = "package"
-        builder = SourcePackageBuilder(name, version)
-        builder.add_upstream_file("README", "Hi\n")
-        builder.add_upstream_file("BUGS")
-        builder.add_default_control()
-        builder.build()
-        dsc = deb822.Dsc(open(builder.dsc_name()).read())
-        self.assertEqual(OneZeroSourceExtractor, SOURCE_EXTRACTORS[dsc['Format']])
-        extractor = OneZeroSourceExtractor(builder.dsc_name(), dsc)
-        try:
-            extractor.extract()
-            unpacked_dir = extractor.extracted_debianised
-            orig_dir = extractor.extracted_upstream
-            self.assertTrue(os.path.exists(unpacked_dir))
-            self.assertTrue(os.path.exists(orig_dir))
-            self.assertTrue(os.path.exists(os.path.join(unpacked_dir,
-                            "README")))
-            self.assertTrue(os.path.exists(os.path.join(unpacked_dir,
-                            "debian", "control")))
-            self.assertTrue(os.path.exists(os.path.join(orig_dir,
-                            "README")))
-            self.assertFalse(os.path.exists(os.path.join(orig_dir,
-                            "debian", "control")))
-            self.assertEquals(1, len(extractor.upstream_tarballs))
-            self.assertEquals(3, len(extractor.upstream_tarballs[0]))
-            self.assertTrue(os.path.exists(extractor.upstream_tarballs[0][0]))
-            self.assertIs(None, extractor.upstream_tarballs[0][1])
-            self.assertIsInstance(
-                extractor.upstream_tarballs[0][2], str) # md5sum
-        finally:
-            extractor.cleanup()
-
-    def test_extract_format1_native(self):
-        version = Version("0.1-1")
-        name = "package"
-        builder = SourcePackageBuilder(name, version, native=True)
-        builder.add_upstream_file("README", "Hi\n")
-        builder.add_upstream_file("BUGS")
-        builder.add_default_control()
-        builder.build()
-        dsc = deb822.Dsc(open(builder.dsc_name()).read())
-        self.assertEqual(OneZeroSourceExtractor, SOURCE_EXTRACTORS[dsc['Format']])
-        extractor = OneZeroSourceExtractor(builder.dsc_name(), dsc)
-        try:
-            extractor.extract()
-            unpacked_dir = extractor.extracted_debianised
-            orig_dir = extractor.extracted_upstream
-            self.assertTrue(os.path.exists(unpacked_dir))
-            self.assertEqual(None, orig_dir)
-            self.assertTrue(os.path.exists(os.path.join(unpacked_dir,
-                            "README")))
-            self.assertTrue(os.path.exists(os.path.join(unpacked_dir,
-                            "debian", "control")))
-        finally:
-            extractor.cleanup()
-
-    def test_extract_format3_native(self):
-        version = Version("0.1")
-        name = "package"
-        builder = SourcePackageBuilder(name, version, native=True,
-                version3=True)
-        builder.add_upstream_file("README", "Hi\n")
-        builder.add_upstream_file("BUGS")
-        builder.add_default_control()
-        builder.build()
-        dsc = deb822.Dsc(open(builder.dsc_name()).read())
-        self.assertEqual(ThreeDotZeroNativeSourceExtractor,
-                SOURCE_EXTRACTORS[dsc['Format']])
-        extractor = ThreeDotZeroNativeSourceExtractor(builder.dsc_name(),
-                dsc)
-        try:
-            extractor.extract()
-            unpacked_dir = extractor.extracted_debianised
-            orig_dir = extractor.extracted_upstream
-            self.assertTrue(os.path.exists(unpacked_dir))
-            self.assertEqual(None, orig_dir)
-            self.assertTrue(os.path.exists(os.path.join(unpacked_dir,
-                            "README")))
-            self.assertTrue(os.path.exists(os.path.join(unpacked_dir,
-                            "debian", "control")))
-        finally:
-            extractor.cleanup()
-
-    def test_extract_format3_quilt(self):
-        version = Version("0.1-1")
-        name = "package"
-        builder = SourcePackageBuilder(name, version, version3=True)
-        builder.add_upstream_file("README", "Hi\n")
-        builder.add_upstream_file("BUGS")
-        builder.add_default_control()
-        builder.build()
-        dsc = deb822.Dsc(open(builder.dsc_name()).read())
-        self.assertEqual(ThreeDotZeroQuiltSourceExtractor,
-                SOURCE_EXTRACTORS[dsc['Format']])
-        extractor = ThreeDotZeroQuiltSourceExtractor(builder.dsc_name(), dsc)
-        try:
-            extractor.extract()
-            unpacked_dir = extractor.extracted_debianised
-            orig_dir = extractor.extracted_upstream
-            self.assertTrue(os.path.exists(unpacked_dir))
-            self.assertTrue(os.path.exists(orig_dir))
-            self.assertTrue(os.path.exists(os.path.join(unpacked_dir,
-                            "README")))
-            self.assertTrue(os.path.exists(os.path.join(unpacked_dir,
-                            "debian", "control")))
-            self.assertTrue(os.path.exists(os.path.join(orig_dir,
-                            "README")))
-            self.assertFalse(os.path.exists(os.path.join(orig_dir,
-                            "debian", "control")))
-            self.assertEquals(1, len(extractor.upstream_tarballs))
-            self.assertEquals(3, len(extractor.upstream_tarballs[0]))
-            self.assertTrue(os.path.exists(extractor.upstream_tarballs[0][0]))
-            self.assertIs(None, extractor.upstream_tarballs[0][1])
-            self.assertIsInstance(
-                extractor.upstream_tarballs[0][2], str) # md5sum
-        finally:
-            extractor.cleanup()
-
-    def test_extract_format3_quilt_bz2(self):
-        version = Version("0.1-1")
-        name = "package"
-        builder = SourcePackageBuilder(name, version, version3=True)
-        builder.add_upstream_file("README", "Hi\n")
-        builder.add_upstream_file("BUGS")
-        builder.add_default_control()
-        builder.build(tar_format='bz2')
-        dsc = deb822.Dsc(open(builder.dsc_name()).read())
-        self.assertEqual(ThreeDotZeroQuiltSourceExtractor,
-                SOURCE_EXTRACTORS[dsc['Format']])
-        extractor = ThreeDotZeroQuiltSourceExtractor(builder.dsc_name(), dsc)
-        try:
-            extractor.extract()
-            unpacked_dir = extractor.extracted_debianised
-            orig_dir = extractor.extracted_upstream
-            self.assertTrue(os.path.exists(unpacked_dir))
-            self.assertTrue(os.path.exists(orig_dir))
-            self.assertTrue(os.path.exists(os.path.join(unpacked_dir,
-                            "README")))
-            self.assertTrue(os.path.exists(os.path.join(unpacked_dir,
-                            "debian", "control")))
-            self.assertTrue(os.path.exists(os.path.join(orig_dir,
-                            "README")))
-            self.assertFalse(os.path.exists(os.path.join(orig_dir,
-                            "debian", "control")))
-            self.assertTrue(os.path.exists(extractor.upstream_tarballs[0][0]))
-        finally:
-            extractor.cleanup()
-
-    def test_extract_format3_quilt_multiple_upstream_tarballs(self):
-        version = Version("0.1-1")
-        name = "package"
-        builder = SourcePackageBuilder(name, version, version3=True,
-                multiple_upstream_tarballs=("foo", "bar"))
-        builder.add_upstream_file("README", "Hi\n")
-        builder.add_upstream_file("BUGS")
-        builder.add_upstream_file("foo/wibble")
-        builder.add_upstream_file("bar/xyzzy")
-        builder.add_default_control()
-        builder.build(tar_format='bz2')
-        dsc = deb822.Dsc(open(builder.dsc_name()).read())
-        self.assertEqual(ThreeDotZeroQuiltSourceExtractor,
-                SOURCE_EXTRACTORS[dsc['Format']])
-        extractor = ThreeDotZeroQuiltSourceExtractor(builder.dsc_name(), dsc)
-        self.addCleanup(extractor.cleanup)
-        self.assertEquals([], extractor.upstream_tarballs)
