@@ -67,6 +67,7 @@ from ..upstream import (
     UpstreamProvider,
     UpstreamSource,
     UScanSource,
+    UScanError,
     extract_tarball_version,
     gather_orig_files,
     new_tarball_name,
@@ -421,6 +422,19 @@ ftp://ftp.samba.org/pub/tdb/tdb-(.+).tar.gz</warnings>
         self.assertEquals("1.2.9",
             UScanSource._xml_report_extract_upstream_version(text))
 
+    def test__xml_report_extract_upstream_version_errors(self):
+        text = b"""
+<dehs>
+blahf =>
+<package>tdb</package>
+<debian-uversion>1.2.8</debian-uversion>
+<errors>something something signature</errors>
+</dehs>
+"""
+        self.assertRaises(
+            UScanError,
+            UScanSource._xml_report_extract_upstream_version, text)
+
 
 class GuessUpstreamRevspecTests(TestCase):
 
@@ -439,11 +453,17 @@ class GuessUpstreamRevspecTests(TestCase):
                  u'tag:v1.2~bzr1200', u'tag:v.1.2~bzr1200'],
                 list(guess_upstream_revspec('foo', '1.2~bzr1200')))
         self.assertEqual(
-                [u'git:abc1def', u'tag:1.2~git20180101.abc1def',
+                [u'git:abc1def', u'date:2018-01-01', u'tag:1.2~git20180101.abc1def',
                  u'tag:foo-1.2~git20180101.abc1def',
                  u'tag:v1.2~git20180101.abc1def',
                  u'tag:v.1.2~git20180101.abc1def'],
                 list(guess_upstream_revspec('foo', '1.2~git20180101.abc1def')))
+        self.assertEqual(
+                [u'git:abc1def', u'date:2018-01-01', u'tag:1.2~git20180101.0.abc1def',
+                 u'tag:foo-1.2~git20180101.0.abc1def',
+                 u'tag:v1.2~git20180101.0.abc1def',
+                 u'tag:v.1.2~git20180101.0.abc1def'],
+                list(guess_upstream_revspec('foo', '1.2~git20180101.0.abc1def')))
         self.assertEqual(
                 [u'date:2018-01-01', u'tag:1.2~git20180101',
                  u'tag:foo-1.2~git20180101',
@@ -811,6 +831,14 @@ class GetRevisionSnapshotTests(TestCase):
     def test_with_svn_snapshot_plus(self):
         self.assertEquals("svn:2424", get_snapshot_revision("0.4.4+svn2424"))
 
+    def test_git(self):
+        self.assertEquals(
+            "date:20190101",
+            get_snapshot_revision("0.4.4+git20190101"))
+        self.assertEquals(
+            "git:abc1def",
+            get_snapshot_revision("0.4.4+git20190101.abc1def"))
+
 
 class TestIsUpstreamTag(TestCase):
 
@@ -894,7 +922,6 @@ class PristineTarSourceTests(TestCaseWithTransport):
     def setUp(self):
         super(PristineTarSourceTests, self).setUp()
         self.tree = self.make_branch_and_tree('unstable')
-        root_id = self.tree.path2id("")
         self.source = PristineTarSource(self.tree.branch)
 
     def test_gbp_tag_format(self):
@@ -907,17 +934,33 @@ upstream-tag = blah-%(version)s
         self.source = PristineTarSource.from_tree(
             self.tree.branch, self.tree)
         upstream_v_no = "0.1"
-        self.assertEqual(self.source.tag_name(upstream_v_no),
-                "blah-" + upstream_v_no)
+        self.assertEqual(
+            self.source.tag_name(upstream_v_no),
+            "blah-" + upstream_v_no)
+
+    def test_gbp_tag_format_unusual(self):
+        self.build_tree_contents([
+            ('unstable/debian/', ),
+            ('unstable/debian/gbp.conf', """\
+[DEFAULT]
+upstream-tag = blah-%(version%~%-)s
+""")])
+        self.source = PristineTarSource.from_tree(
+            self.tree.branch, self.tree)
+        self.assertEqual(
+            self.source.tag_name("0.1~1"),
+            "blah-0.1-1")
 
     def test_upstream_tag_name(self):
         upstream_v_no = "0.1"
-        self.assertEqual(self.source.tag_name(upstream_v_no),
-                "upstream-" + upstream_v_no)
+        self.assertEqual(
+            self.source.tag_name(upstream_v_no),
+            "upstream-" + upstream_v_no)
 
     def test_tag_name_distro(self):
-        self.assertEquals(self.source.tag_name("0.3", distro="ubuntu"),
-                "upstream-ubuntu-0.3")
+        self.assertEquals(
+            self.source.tag_name("0.3", distro="ubuntu"),
+            "upstream-ubuntu-0.3")
 
     def test_version(self):
         self.assertEquals([
