@@ -26,9 +26,10 @@ import sys
 import tarfile
 import tempfile
 
-from ....errors import NoSuchFile
+from ....errors import BzrError, NoSuchFile
 from .... import osutils
 from ....export import export
+from ....sixish import PY3
 from ....trace import (
     note,
     warning,
@@ -166,6 +167,14 @@ class AptSource(UpstreamSource):
         return True
 
 
+class UScanError(BzrError):
+
+    _fmt = "UScan failed to run: %(errors)s."
+
+    def __init__(self, errors):
+        self.errors = errors
+
+
 class UScanSource(UpstreamSource):
     """Upstream source that uses uscan."""
 
@@ -192,8 +201,17 @@ class UScanSource(UpstreamSource):
     def _run_dehs_uscan(args, cwd):
         p = subprocess.Popen(
             ["uscan", "--dehs"] + args,
-            stdout=subprocess.PIPE, cwd=cwd)
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd)
         (stdout, stderr) = p.communicate()
+        if b'</dehs>' not in stdout:
+            error = stderr.decode()
+            if error.startswith('uscan error '):
+                error = error[len('uscan error '):]
+            raise UScanError(error)
+        if PY3:
+            sys.stderr.write(stderr.decode())
+        else:
+            sys.stderr.write(stderr)
         return stdout, p.returncode
 
     @staticmethod
@@ -202,9 +220,16 @@ class UScanSource(UpstreamSource):
         for m in re.finditer(b"<warnings>(.*)</warnings>", text):
             warning(unescape(m.group(1).decode()))
 
+    @staticmethod
+    def _xml_report_extract_errors(text):
+        from xml.sax.saxutils import unescape
+        for m in re.finditer(b"<errors>(.*)</errors>", text):
+            raise UScanError(unescape(m.group(1).decode()))
+
     @staticmethod
     def _xml_report_extract_upstream_version(text):
         UScanSource._xml_report_extract_warnings(text)
+        UScanSource._xml_report_extract_errors(text)
         from xml.sax.saxutils import unescape
         # uscan --dehs's output isn't well-formed XML, so let's fall back to
         # regexes instead..
@@ -216,6 +241,7 @@ class UScanSource(UpstreamSource):
     @staticmethod
     def _xml_report_extract_target_paths(text):
         UScanSource._xml_report_extract_warnings(text)
+        UScanSource._xml_report_extract_errors(text)
         from xml.sax.saxutils import unescape
         return [
             unescape(m.group(1).decode())
