@@ -170,7 +170,8 @@ def upstream_version_add_revision(upstream_branch, version_string, revid):
     if m and gitid:
         return "%s%sgit%s.%s" % (m.group(1), m.group(2), gitdate, gitid)
 
-    m = re.match(r"^(.*)([\+~])git(\d{8})\.(\d+)\.([a-f0-9]{7})$", version_string)
+    m = re.match(r"^(.*)([\+~])git(\d{8})\.(\d+)\.([a-f0-9]{7})$",
+                 version_string)
     if m and gitid:
         if gitdate == m.group(3):
             snapshot = int(m.group(4)) + 1
@@ -220,6 +221,13 @@ def get_snapshot_revision(upstream_version):
     match = re.search("(?:~|\\+)svn([0-9]+)$", upstream_version)
     if match is not None:
         return "svn:%s" % match.groups()[0]
+    match = re.match(r"^(.*)([\+~])git(\d{8})\.([a-f0-9]{7})$",
+                     upstream_version)
+    if match:
+        return "git:%s" % match.group(4)
+    match = re.match(r"^(.*)([\+~])git(\d{8})$", upstream_version)
+    if match:
+        return "date:%s" % match.group(3)
     return None
 
 
@@ -275,9 +283,16 @@ def guess_upstream_revspec(package, version):
     if "+bzr" in version or "~bzr" in version:
         yield "revno:%s" % re.match(".*[~+]bzr(\\d+).*", version).group(1)
     if "+git" in version or "~git" in version:
-        m = re.match(".*[~+]git([0-9]{8})\\.([0-9a-f]{7}).*", version)
+        m = re.match(
+            ".*[~+]git([0-9]{4})([0-9]{2})([0-9]{2})\\.([0-9a-f]{7}).*",
+            version)
+        if not m:
+            m = re.match(
+                ".*[~+]git([0-9]{4})([0-9]{2})([0-9]{2})\\.[0-9+]\\.([0-9a-f]{7}).*",
+                version)
         if m:
-            yield "git:%s" % m.group(2)
+            yield "git:%s" % m.group(4)
+            yield "date:%s-%s-%s" % (m.group(1), m.group(2), m.group(3))
         else:
             m = re.match(".*[~+]git([0-9]{4})([0-9]{2})([0-9]{2}).*", version)
             if m:
