@@ -64,7 +64,7 @@ from .config import (
     BUILD_TYPE_NORMAL,
     )
 from .errors import (
-    MissingChangelogError,
+    BzrError,
     AddChangelogError,
     InconsistentSourceFormatError,
     NoPreviousUpload,
@@ -73,6 +73,14 @@ from .errors import (
     UnparseableChangelog,
     )
 
+
+class MissingChangelogError(BzrError):
+    _fmt = 'Could not find changelog at %(location)s in tree.'
+
+    def __init__(self, locations):
+        BzrError.__init__(self, location=locations)
+
+
 _DEBIAN_RELEASES = None
 _UBUNTU_RELEASES = None
 
@@ -559,22 +567,21 @@ def subprocess_setup():
     signal.signal(signal.SIGPIPE, signal.SIG_DFL)
 
 
-def debuild_config(tree, working_tree):
+def debuild_config(tree):
     """Obtain the Debuild configuration object.
 
     :param tree: A Tree object, can be a WorkingTree or RevisionTree.
-    :param working_tree: Whether the tree is a working tree.
     """
     config_files = []
     user_config = None
-    if (working_tree and tree.has_filename(new_local_conf)):
+    if tree.has_filename(new_local_conf):
         if not tree.is_versioned(new_local_conf):
             config_files.append((tree.get_file(new_local_conf), True,
                         "local.conf"))
         else:
             warning('Not using configuration from %s as it is versioned.',
                     new_local_conf)
-    if (working_tree and tree.has_filename(local_conf)):
+    if tree.has_filename(local_conf):
         if not tree.is_versioned(local_conf):
             config_files.append((tree.get_file(local_conf), True,
                         "local.conf"))
@@ -797,6 +804,27 @@ def dput_changes(path):
     subprocess.check_call(["dput", changes_file], cwd=bd)
 
 
-def debsign(path):
+def debsign(path, keyid=None):
     (bd, changes_file) = os.path.split(path)
-    subprocess.check_call(["debsign", changes_file], cwd=bd)
+    args = ["debsign"]
+    if keyid:
+        args.append("-k%s" % keyid)
+    args.append(changes_file)
+    subprocess.check_call(args, cwd=bd)
+
+
+def changes_filename(package, version, arch):
+    non_epoch_version = version.upstream_version
+    if version.debian_version is not None:
+        non_epoch_version += "-%s" % version.debian_version
+    return "%s_%s_%s.changes" % (package,
+            non_epoch_version, arch)
+
+
+def get_build_architecture():
+    try:
+        return subprocess.check_output(
+            ['dpkg-architecture', '-qDEB_BUILD_ARCH']).strip().decode()
+    except subprocess.CalledProcessError as e:
+        raise BzrError(
+            "Could not find the build architecture: %s" % e)
